@@ -1,25 +1,61 @@
 """
-This module, 'class_utils.py', contains utility functions for dealing with classes, objects, and modules.
-These include functions to:
+Sheet: Class Utilities
 
-1. Manipulate global variables.
-2. Fetch and check object types and their membership in a module.
-3. Fetch and inspect function signatures within a module.
-4. Call functions with supplied arguments.
-5. Convert layout to components.
-6. Retrieve directory of a module.
+Module: class_utils.py
+Part of: abstract_utilities module
+Author: putkoff
+Date: 05/31/2023
+Version: 0.1.2
+
+This module contains utility functions tailored for handling classes, objects, and modules. It encapsulates operations like:
+1. Fetching and checking object types.
+2. Manipulating global variables.
+3. Checking object membership in a module.
+4. Inspecting function signatures and their arguments within a module.
+5. Calling functions with supplied arguments.
+6. Converting layout definitions into components.
+7. Retrieving attributes and methods of a module.
+
+Function Overview:
+------------------
+- get_type_list: Get a list of common Python types.
+- remove_key: Remove a specified key from a dictionary.
+- get_module_obj: Retrieve an object from a given module.
+- spec_type_mod: Check if an object has a specific type.
+- get_type_mod: Retrieve the type name of a given object.
+- is_module_obj: Check if an object is part of a module.
+- inspect_signature: Fetch the signature of a specified function.
+- get_parameter_defaults: Fetch the default parameter values for a function.
+- convert_layout_to_components: Convert a layout definition to its component representation.
+- get_dir: List all attributes and methods of a module.
+- get_proper_args: Call a function using either positional or keyword arguments.
+- get_fun: Parse a dictionary to retrieve a function and call it.
+- if_none_change: Replace a None object with a default value.
+- call_functions: Call a specified function or method using provided arguments.
+- process_args: Evaluate and process nested function calls in arguments.
+- has_attribute: Check if a function exists in a module.
+- mk_fun: Print a statement indicating the existence of a function in a module.
+
+Dependencies:
+-------------
+- inspect
+- json
+
+Each function is furnished with its own docstring that elaborates on its purpose, expected inputs, and outputs.
 
-Each function includes a docstring to further explain its purpose, input parameters, and return values.
 """
 import inspect
 import json
-def get_type_list()->list:
+def get_type_list() -> list:
+    """Get a list of common Python types."""
     return ['None','str','int','float','bool','list','tuple','set','dict','frozenset','bytearray','bytes','memoryview','range','enumerate','zip','filter','map','property','slice','super','type','Exception','object']
+
 def remove_key(js: dict, key: any) -> dict:
     """Remove a key from a dictionary. If the key is not present,
     no action is taken."""
     js.pop(key, None)
     return js
+
 def get_module_obj(instance: any, obj: any):
     """
     Retrieves an object from a module.
@@ -33,7 +69,6 @@ def get_module_obj(instance: any, obj: any):
     """
     return getattr(module, obj)
 
-
 def spec_type_mod(obj: any, st: str) -> bool:
     """
     Checks if an object has a specific type.
@@ -49,7 +84,6 @@ def spec_type_mod(obj: any, st: str) -> bool:
         return True
     return False
 
-
 def get_type_mod(obj: any) -> str:
     """
     Retrieves the type of an object.
@@ -67,7 +101,6 @@ def get_type_mod(obj: any) -> str:
             return typ
     return "NoneType"
 
-
 def is_module_obj(instance: any, obj: str) -> bool:
     """
     Checks if an object is part of a module.
@@ -85,6 +118,7 @@ def is_module_obj(instance: any, obj: str) -> bool:
         return True
     except:
         return False
+
 def inspect_signature(instance: any, function: str):
     """
     Inspects the signature of a function.
@@ -97,6 +131,7 @@ def inspect_signature(instance: any, function: str):
         inspect.Signature: The signature of the function.
     """
     return inspect.signature(get_module_obj(instance, function))
+
 def get_parameter_defaults(module, function):
     """
     Retrieves the default parameter values of a function.
@@ -112,6 +147,7 @@ def get_parameter_defaults(module, function):
     if signature is None:
         return {}
     return json.dumps(remove_key({param_name: param.default if param.default != inspect.Parameter.empty else None for param_name, param in signature.parameters.items()}, 'icon'))
+
 def convert_layout_to_components(instance: any = None, component: str = 'function', js: dict = {}):
     """
     Converts a layout to components.
@@ -131,7 +167,6 @@ def convert_layout_to_components(instance: any = None, component: str = 'functio
             jsN[key] = js[key]
     return call_functions_hard(instance, component, **jsN)
 
-
 def get_dir(mod):
     """
     Retrieves the directory of a module.
@@ -143,8 +178,11 @@ def get_dir(mod):
         list: The list of attributes and methods in the module.
     """
     return dir(mod)
+
 def get_proper_args(function,args:(dict or list)={}):
+    """Call a function with either positional or keyword arguments based on the provided args type."""
     return function(*args) if isinstance(args, list) else function(**args)
+
 def get_fun(js):
     """
     Retrieves and calls a function with the given parameters.
@@ -175,7 +213,9 @@ def get_fun(js):
     # Filter arguments to only those accepted by the function
     filtered_args = {k: v for k, v in function_args.items() if k in valid_keys}
     return call_functions(function_name, filtered_args, instance, glob)
-def if_none_change(obj,default):
+
+def if_none_change(obj, default):
+    """Return a default value if the provided object is None."""
     if obj == None:
         obj = default
     return obj
@@ -201,6 +241,7 @@ def call_functions(function_name: str, args: dict = {}, instance=None, glob:(dic
     else:
         # Calls function from globals
         return get_proper_args(glob[function_name],args)
+    
 def process_args(args):
     """
     Processes the arguments for a function, replacing nested function calls with their results.
@@ -211,7 +252,6 @@ def process_args(args):
     Returns:
         dict: A dictionary of processed arguments.
     """
-    
     for key, value in args.items():
         # check if value is a dict and has a 'type' key with value 'get'
         if isinstance(value, dict) and value.get('type') == 'get':
@@ -222,13 +262,60 @@ def process_args(args):
             # call the function and replace the arg with its result
             args[key] = call_functions(function_name, function_args, instance, glob)
     return args
-# Check if the function exists
-def has_attribute(module,function):
+
+def has_attribute(module, function):
+    """Check if a specific function exists in a given module."""
     try:
         bool_it = hasattr(module, function)
     except:
         bool_it =None
     return bool_it
+
+def get_all_functions_for_instance(instance):
+    """
+    Retrieves all callable methods/functions of an object instance.
+
+    Args:
+        instance: The object instance for which to retrieve methods/functions.
+
+    Returns:
+        list: A list of method/function names that are callable on the given instance.
+    """
+    return [method for method in dir(instance) if callable(getattr(instance, method))]
+def get_all_params(instance, function_name):
+    """
+    Retrieves information about the parameters of a callable method/function of an instance.
+
+    Args:
+        instance: The object instance containing the method.
+        function_name (str): The name of the method/function to inspect.
+
+    Returns:
+        dict: A dictionary containing parameter information, including names, defaults, kinds, and required parameters.
+    """
+    # Use getattr() to get the method by name
+    method = getattr(instance, function_name, None)
+    
+    if callable(method):
+        # Now you have the method, and you can inspect it
+        func_signature = inspect.signature(method)
+        parameters = func_signature.parameters
+        
+        params = {"names": [], "defaults": [], "kinds": [], "required": []}
+        
+        for param_name, param in parameters.items():
+            params["names"].append(param_name)
+            params["defaults"].append(param.default)
+            params["kinds"].append(param.kind)
+            
+            if param.default == inspect._empty:
+                params["required"].append(param_name)
+        
+        return params
+    else:
+        print(f"{function_name} is not a callable method of the instance.")
+        return None
+    
 def mk_fun(module,function):
     """
     Checks if a function exists in a given module.
